# 21 - Pentesting FTP

## Basic Information

The **File Transfer Protocol \(FTP**\) is a standard network protocol used for the transfer of computer files between a client and server on a computer network.  
It is a **plain-text** protocol that uses as **new line character `0x0d 0x0a`** so it's important to **connect using telnet** instead of nc.

**Default Port:** 21

```text
PORT   STATE SERVICE
21/tcp open  ftp
```

## Enumeration

### **Banner Grabbing**

```bash
telnet -vn <IP> 21
openssl s_client -connect crossfit.htb:21 -starttls ftp #Get certificate if any
```

### Unauth enum

You can us the commands `HELP` and `FEAT` to obtain some information of the FTP server:

```text
HELP
214-The following commands are recognized (* =>'s unimplemented):
214-CWD     XCWD    CDUP    XCUP    SMNT*   QUIT    PORT    PASV    
214-EPRT    EPSV    ALLO*   RNFR    RNTO    DELE    MDTM    RMD     
214-XRMD    MKD     XMKD    PWD     XPWD    SIZE    SYST    HELP    
214-NOOP    FEAT    OPTS    AUTH    CCC*    CONF*   ENC*    MIC*    
214-PBSZ    PROT    TYPE    STRU    MODE    RETR    STOR    STOU    
214-APPE    REST    ABOR    USER    PASS    ACCT*   REIN*   LIST    
214-NLST    STAT    SITE    MLSD    MLST    
214 Direct comments to root@drei.work
FEAT
211-Features:
 PROT
 CCC
 PBSZ
 AUTH TLS
 MFF modify;UNIX.group;UNIX.mode;
 REST STREAM
 MLST modify*;perm*;size*;type*;unique*;UNIX.group*;UNIX.mode*;UNIX.owner*;
 UTF8
 EPRT
 EPSV
 LANG en-US
 MDTM
 SSCN
 TVFS
 MFMT
 SIZE
211 End
```

### Connections

In **Active FTP** the FTP **client** first **initiates** the control **connection** from its port N to FTP Servers command port – port 21. The **client** then **listens** to port **N+1** and sends the port N+1 to FTP Server. FTP **Server** then **initiates** the data **connection**, from **its port M to the port N+1** of the FTP Client.

But, if the FTP Client has a firewall setup that controls the incoming data connections from outside, then active FTP may be a problem. And, a feasible solution for that is Passive FTP.

In **Passive FTP**, the client initiates the control connection from its port N to the port 21 of FTP Server. After this, the client issues a **passv comand**. The server then sends the client one of its port number M. And the **client** **initiates** the data **connection** from **its port P to port M** of the FTP Server.

Source: [https://www.thesecuritybuddy.com/vulnerabilities/what-is-ftp-bounce-attack/](https://www.thesecuritybuddy.com/vulnerabilities/what-is-ftp-bounce-attack/)

### Anonymous login

_anonymous : anonymous  
anonymous :   
ftp : ftp_

```bash
ftp <IP>
>anonymous
>anonymous
>ls -a # List all files (even hidden) (yes, they could be hidden)
>binary #Set transmission to binary instead of ascii
>ascii #Set transmission to ascii instead of binary
>bye #exit
```

### [Brute force](../../brute-force.md#ftp)

Here you can find a nice list with default ftp credentials: [https://github.com/danielmiessler/SecLists/blob/master/Passwords/Default-Credentials/ftp-betterdefaultpasslist.txt](https://github.com/danielmiessler/SecLists/blob/master/Passwords/Default-Credentials/ftp-betterdefaultpasslist.txt)

### Automated

Anon login and bounce FTP checks are perform by default by nmap with **-sC** option.

### Shodan

* `ftp`
* `port:21`

## Browser connection

You can connect to a FTP server using a browser \(like Firefox\) using a URL like: 

```bash
ftp://anonymous:anonymous@10.10.10.98
```

Note that if a **web application** is sending data controlled by a user **directly to a FTP server** you can send double URL encode `%0d%0a` \(in double URL encode this is `%250d%250a`\) bytes and make the **FTP server perform arbitrary actions**. One of this possible arbitrary actions is to download content from a users controlled server, perform port scanning or try to talk to other plain-text based services \(like http\).

## Download all files from FTP

```bash
wget -m ftp://anonymous:anonymous@10.10.10.98 #Donwload all
wget -m --no-passive ftp://anonymous:anonymous@10.10.10.98 #Download all
```

## Some FTP commands

* `USER username`
* `PASS password`
* `HELP` The server indicates which commands are supported
* `PORT 127,0,0,1,0,80`This will indicate the FTP server to establish a connection with the IP 127.0.0.1 in port 80 \(_you need to put the 5th char as "0" and the 6th as the port in decimal or use the 5th and 6th to express the port in hex_\).
* `EPRT |2|127.0.0.1|80|`This will indicate the FTP server to establish a TCP connection \(_indicated by "2"_\) with the IP 127.0.0.1 in port 80. This command **supports IPv6**.
* `LIST` This will send the list of files in current folder
* `APPE /path/something.txt` This will indicate the FTP to store the data received from a **passive** connection or from a **PORT/EPRT** connection to a file. If the filename exists, it will append the data.
* `STOR /path/something.txt` Like `APPE` but it will overwrite the files
* `STOU /path/something.txt` Like `APPE`, but if exists it won't do anything.
* `RETR /path/to/file` A passive or a port connection must be establish. Then, the FTP server will send the indicated file through that connection
* `REST 6` This will indicate the server that next time it send something using `RETR` it should start in the 6th byte.
* `TYPE i` Set transfer to binary
* `PASV` This will open a passive connection and will indicate the user were he can connects 

![](../../.gitbook/assets/image%20%28184%29.png)

## FTPBounce attack

Some FTP servers allow the command PORT. This command can be used to indicate to the server that you wants to connect to other FTP server at some port. Then, you can use this to scan which ports of a host are open through a FTP server.

[**Learn here how to abuse a FTP server to scan ports.**](ftp-bounce-attack.md)\*\*\*\*

You could also abuse this behaviour to make a FTP server interact with other protocols. You could **upload a file containing an HTTP request** and make the vulnerable FTP server **send it to an arbitrary HTTP server** \(_maybe to add a new admin user?_\) or even upload a FTP request and make the vulnerable FTP server download a file for a different FTP server.   
The theory is easy:

1. **Upload the request \(inside a text file\) to the vulnerable server.** Remember that if you want to talk with another HTTP or FTP server you need to change lines with `0x0d 0x0a`
2. **Use `REST X` to avoid sending the characters you don't want to send** \(maybe to upload the request inside the file you needed to put some image header at the begging\)
3. **Use `PORT`to connect to the arbitrary server and service** 
4. **Use `RETR`to send the saved request to the server.** 

Its highly probably that this **will throw an error like** _**Socket not writable**_ **because the connection doesn't last enough to send the data with `RETR`**. Suggestions to try to avoid that are:

* If you are sending an HTTP request, **put the same request one after another** until **~0.5MB** at least. Like this: 

{% file src="../../.gitbook/assets/posts \(1\).txt" caption="posts.txt" %}

* Try to **fill the request with "junk" data relative to the protocol** \(talking to FTP maybe just junk commands or repeating the `RETR`instruction to get the file\)
* Just **fill the request with a lot of null characters or others** \(divided on lines or not\)

Anyway, here you have an [old example about how to abuse this to make a FTP server download a file from a different FTP server.](ftp-bounce-download-2oftp-file.md)

## Filezilla Server Vulnerability

**FileZilla** usually **binds** to **local** an **Administrative service** for the **FileZilla-Server** \(port 14147\). If you can create a **tunnel** from **your machine** to access this port, you can **connect** to **it** using a **blank password** and **create** a **new user** for the FTP service.

## Config files

```text
ftpusers
ftp.conf
proftpd.conf
```

